#!/usr/bin/bash
#
# v1.0 10.02.2025 Jochen Bandhauer (www.jbitc.de): Initial Release
# v1.1 13.02.2025 Jochen Bandhauer (www.jbitc.de): Cleanup, quit when dnf is running, performance improvement
#
# moves all local cached rpm files to a shared location and creates a symbolic link to the moved file
# package caching needs to be enabled by setting keepcache=1 in /etc/dnf/dnf.conf
#
# enable debug output: /usr/local/bin/pmrc.sh 1
#

### step 1: initialization
# set debug to 1 to enable debug output
debug=$1
# set the path where the shared rpms should reside
spath=/sw/rpmcache
date=`date`

### step 2: quit when dnf is currently running
if [ -f /var/cache/dnf/metadata_lock.pid ] || [ -f /var/cache/dnf/download_lock.pid ]; then
  echo $date 'Quitting because dnf is currently running. The lockfile is:' `ls /var/cache/dnf/*_lock.pid`
  exit 1
fi

### step 3: move local rpm file from the cache to the shared location
[[ $debug -eq 1 ]] && echo $date 'Starting step 3'
for source in `find /var/cache/dnf/ -type f -name '*.rpm'`; do

  # define variables
  path=`echo $source|cut -d "/" -f -6`
  shortpath=`echo $path|cut -d "/" -f 5-`
  filename=`echo $source|cut -d "/" -f 7`

  # create destination path
  if [ ! -d $spath/$shortpath ]; then
    [[ $debug -eq 1 ]] && echo $date 'Creating path in shared location' $spath/$shortpath
    mkdir -p $spath/$shortpath
  fi

  # move rpm file if it doesn't exist
  if [ -f $spath/$shortpath/$filename ]; then
    [[ $debug -eq 1 ]] && echo $date 'Delete rpm file from local cache:' $path/$filename
    rm -f $path/$filename
  else
    [[ $debug -eq 1 ]] && echo $date 'Moving rpm to shared location:' $path/$filename
    mv $path/$filename $spath/$shortpath/
  fi

done

### step 4: create a symbolic link in the local cache directory for all rpm files on the shared location
[[ $debug -eq 1 ]] && echo $date 'Starting step 4'
for sharedrpm in `find $spath/ -type f -name '*.rpm'`; do

  # define variables
  tmp=${sharedrpm#$spath/}   # remove $spath
  shortpath=${tmp%/*rpm}     # remove rpm filename
  filename=${sharedrpm##/*/} # extract rpm filename

  # create directory
  if [ ! -d /var/cache/dnf/$shortpath ]; then
    [[ $debug -eq 1 ]] && echo $date 'Creating directory for link:' /var/cache/dnf/$shortpath
    mkdir -p /var/cache/dnf/$shortpath ]
  fi

  # create symbolic link
  if [ ! -h /var/cache/dnf/$shortpath/$filename ]; then
    [[ $debug -eq 1 ]] && echo $date 'Creating link:' /var/cache/dnf/$shortpath/$filename
    ln -s $spath/$shortpath/$filename /var/cache/dnf/$shortpath/$filename
  fi

done
